from bookshelf_common.errors import CustomError
class NotFoundError(CustomError):
    status = 404
    message = 'Not found'

    def __init__(self):
        self.message = 'Not found'
        super().__init__(self.message)


from typing import List, Dict

class CustomError(Exception):
    status = 400
    message = None
    def __init__(self, message=None, status=None):
        if status is not None:
            self.status = status
        self.message = message
        super().__init__(self.message)

    @property
    def serialize(self) -> List[Dict]:
        return [{'message': self.message}]


from bookshelf_common.errors.custom_error import CustomError
from bookshelf_common.errors.bad_request_error import BadRequestError
from bookshelf_common.errors.not_authorized_error import NotAuthorizedError
from bookshelf_common.errors.not_found_error import NotFoundError
from bookshelf_common.errors.request_validation_error import RequestValidationError


from bookshelf_common.events.objects.book_exchange import BookExchange
from bookshelf_common.events.objects.book_status_exchange import BookStatusExchange
from bookshelf_common.events.objects.user_exchange import UserExchange
from bookshelf_common.events.objects.order_exchange import OrderExchange
from bookshelf_common.events.objects.book_sold import BookSold
from bookshelf_common.events.objects.queue_list import QueueList


from bookshelf_common.events import create_connection

def publish_message(obj, exchange_type='fanout', channel=None, connection=None, **kwargs):
  if channel is None:
    channel, connection = create_connection(**kwargs)
  
  message_body = obj.message_body

  # Create fanout exchange
  channel.exchange_declare(exchange=obj.subject, exchange_type=exchange_type)

  # Publish the message to the fanout exchange
  channel.basic_publish(exchange=obj.subject, routing_key='', body=message_body)
  
  print(f" [x] Sent user creation message: {message_body}")
  # Close the connection
  connection.close()



from dataclasses import dataclass, asdict
import json

@dataclass
class BookExchange:
  id: int
  title: str
  slug: str
  novelist: str
  description: str
  price: int
  status: str
  genre: str
  user_id: int
  version: int
  action: str = 'create'

  @property
  def subject(self) -> str:
    return 'book_exchange'
  
  @property
  def message_body(self) -> str:
    return json.dumps(asdict(self))

  @classmethod
  def parse_message(cls, msg):
    data = json.loads(msg)
    return cls(**data)


from dataclasses import dataclass, asdict
import json

@dataclass
class BookStatusExchange:
  id: int
  status: str
  status_msg: str
  msg_type: str

  @property
  def subject(self) -> str:
    return 'book_status_exchange'
  
  @property
  def message_body(self) -> str:
    return json.dumps(asdict(self))
  
  @classmethod
  def parse_message(cls, msg):
    data = json.loads(msg)
    return cls(**data)


from bookshelf_common.errors import CustomError
class BadRequestError(CustomError):
    status = 400
    message = None
    
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)